package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.album.entity.dao.AlbumCatalogue;
import com.redxun.knowledge.album.entity.dao.AlbumCatalogueKnowledge;
import com.redxun.knowledge.album.entity.dao.AlbumVersion;
import com.redxun.knowledge.album.entity.vo.AlbumCatalogueVo;
import com.redxun.knowledge.album.entity.vo.AlbumKnowledgeVo;
import com.redxun.knowledge.album.mapper.AlbumCatalogueKnowledgeMapper;
import com.redxun.knowledge.album.mapper.AlbumCatalogueMapper;
import com.redxun.knowledge.album.mapper.AlbumVersionMapper;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.service.KnowledgeCategoryServiceImpl;
import com.redxun.knowledge.service.KnowledgeServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* [业务--专辑版本目录表]业务服务类
*/
@Service
public class AlbumCatalogueServiceImpl extends SuperServiceImpl<AlbumCatalogueMapper, AlbumCatalogue> implements BaseService<AlbumCatalogue> {

    @Resource
    private AlbumCatalogueMapper albumCatalogueMapper;

    @Autowired
    private AlbumCatalogueKnowledgeMapper albumCatalogueKnowledgeMapper;

    @Autowired
    private KnowledgeServiceImpl knowledgeService;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Autowired
    private UserService userService;

    @Autowired
    private AlbumVersionMapper albumVersionMapper;

    @Override
    public BaseDao<AlbumCatalogue> getRepository() {
        return albumCatalogueMapper;
    }


    @Override
    public IPage query(QueryFilter queryFilter) {
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        IPage page = albumCatalogueMapper.findAllAlbumCatalogue(queryFilter.getPage(), queryFilter.getParams(), params);
        if (page.getRecords() != null){
            //查询版本对应信息(生效中 可移除知识)
            String  versionId = (String) queryFilter.getParams().get("versionId");
            AlbumVersion albumVersion = albumVersionMapper.selectById(versionId);
            //判断该专辑是否有审核中或驳回的版本
            List<AlbumVersion> albumVersions = albumVersionMapper.
                    selectList(new LambdaQueryWrapper<AlbumVersion>().eq(AlbumVersion::getAlbumId,albumVersion.getAlbumId()));
            List<AlbumCatalogueVo> albumCatalogueVoList = (List<AlbumCatalogueVo>) page.getRecords().parallelStream().map(e -> {
                AlbumCatalogueVo albumCatalogueVo = new AlbumCatalogueVo();
                BeanUtils.copyProperties(e, albumCatalogueVo);
                if (albumVersion.getState() == 2){
                    List<String> actionFlag = new ArrayList<>();
                    actionFlag.add("delete");
                    albumCatalogueVo.setActionFlag(actionFlag);
                }
                if (albumVersions.stream().filter(h -> h.getState() == 1).count() > 0 || albumVersions.stream().filter(g -> g.getState() == 4).count() > 0){
                    albumCatalogueVo.setActionFlag(new ArrayList<>());
                }
                return albumCatalogueVo;
            }).collect(Collectors.toList());
            //目录下的知识内容
            albumCatalogueVoList.forEach(albumCatalogueVo -> {
                //查询对应的知识Id
                List<AlbumCatalogueKnowledge> albumCatalogueKnowledgeList = albumCatalogueKnowledgeMapper.
                        findAllAlbumCatalogueKnowledgeList(albumCatalogueVo.getPkId());
                //查询对应的知识
                List<AlbumKnowledgeVo> albumKnowledgeVoList = albumCatalogueKnowledgeList.parallelStream().map(albumCatalogueKnowledge -> {
                    AlbumKnowledgeVo albumKnowledgeVo = new AlbumKnowledgeVo();
                    Knowledge knowledge = knowledgeService.get(albumCatalogueKnowledge.getKnowledgeId());
                    if (knowledge != null) {
                        albumKnowledgeVo.setKnowledgeId(knowledge.getPkId());
                        //查询分类信息
                        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(knowledge.getCategoryId());
                        String category = knowledgeCategoryAdminVo.getName();
                        if (knowledgeCategoryAdminVo.getParents() != null){
                            category = knowledgeCategoryAdminVo.getParents().getName() + "|" + category;
                            if (knowledgeCategoryAdminVo.getParents().getParents() != null){
                                category = knowledgeCategoryAdminVo.getParents().getParents().getName() + "|" +
                                        knowledgeCategoryAdminVo.getParents().getName() + "|" + category;
                            }
                        }
                        albumKnowledgeVo.setCategory(category);
                        albumKnowledgeVo.setTitle(knowledge.getTitles());
                        albumKnowledgeVo.setAuthor(knowledge.getAuthor());
                        albumKnowledgeVo.setAuthorHead((String) userService.querySexAndPhoto(knowledge.getCreateBy()).get("photo"));
                        albumKnowledgeVo.setCreateTime(knowledge.getCreateTime());
                    }
                    return albumKnowledgeVo;
                }).collect(Collectors.toList());
                albumCatalogueVo.setContents(albumKnowledgeVoList);
            });
            page.setRecords(albumCatalogueVoList);
        }
        return page;
    }

    /**
     * 清除该版本下原有目录(同时清除目录下的知识)
     * @param versionId
     */
    public void deleteByVersionId(String versionId) {
        albumCatalogueMapper.deleteByVersionId(versionId);
        //清除知识
        List<AlbumCatalogue> albumCatalogueList = albumCatalogueMapper.
                selectList(new LambdaQueryWrapper<AlbumCatalogue>().eq(AlbumCatalogue::getVersionId, versionId));
        List<String> albumCatalogueIdList = albumCatalogueList.parallelStream().map(AlbumCatalogue::getPkId).collect(Collectors.toList());
        //批量删除知识
        if (CollectionUtils.isNotEmpty(albumCatalogueIdList)){
            albumCatalogueKnowledgeMapper.deleteByCatalogueId(albumCatalogueIdList);
        }

    }
}
