package com.redxun.knowledge.controller;

import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.knowledge.analysis.entity.dto.SystemLogDto;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.CommonEntity;
import com.redxun.knowledge.entity.vo.DicVo;
import com.redxun.knowledge.entity.vo.PositionSequenceVo;
import com.redxun.knowledge.service.CommonServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 系统内容相关
 * 比如e-learning
 */

@Slf4j
@RestController
@RequestMapping("/api-knowledge/common")
@Api(tags = "业务-系统相关")
@ClassDefine(title = "业务--系统相关", alias = "CommonController", path = "/api-knowledge/common", packages = "", packageName = "子系统名称")
public class CommonController extends BaseController<CommonEntity> {

    @Resource
    CommonServiceImpl systemService;

    @Override
    public BaseService getBaseService() {
        return null;
    }

    @Override
    public String getComment() {
        return null;
    }

    @ApiOperation(value = "查询职位-职系")
    @GetMapping("/position/sequences")
    public JsonResult queryPositionSequence() {
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<PositionSequenceVo> positionSequences = systemService.queryPositionSequence();
        jsonResult.setData(positionSequences);
        return jsonResult;
    }

    @ApiOperation(value = "查询职位-职级")
    @GetMapping("/position/grades")
    public JsonResult queryPositionGrade() {
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<String> positionGrades = systemService.queryPositionGrade();
        jsonResult.setData(positionGrades);
        return jsonResult;
    }

    @ApiOperation(value = "通过字典别名获取字典信息")
    @GetMapping("dic")
    public JsonResult queryDic(@RequestParam(value = "alias",defaultValue = "KNOWLEDGE_MAP") String alias) {
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<DicVo> dicVoList = systemService.queryDic(alias);
        jsonResult.setData(dicVoList);
        return jsonResult;
    }

    @ApiOperation("获取当前登陆人信息")
    @GetMapping("loginUserInfo")
    public JsonResult loginUserInfo(){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        Map<String,Object> result = systemService.loginUserInfo();
        return jsonResult.setData(result);
    }

    @ApiOperation("用户浏览估计上报")
    @PostMapping("pageTrack")
    public JsonResult pageTrack(@RequestBody SystemLogDto systemLogDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        systemService.pageTrack(systemLogDto);
        return jsonResult.setData("");
    }

}
