
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;

import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.knowledge.entity.dao.KnowledgeApprovalNode;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeDto;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalVo;
import com.redxun.knowledge.service.KnowledgeApprovalNodeServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/knowledgeApprovalNode")
@Api(tags = "业务--知识审批节点表")
@ClassDefine(title = "业务--知识审批节点表", alias = "KnowledgeApprovalNodeController", path = "/knowledge/knowledgeApprovalNode", packages = "", packageName = "子系统名称")
public class KnowledgeApprovalNodeController extends BaseController<KnowledgeApprovalNode> {

    @Autowired
    KnowledgeApprovalNodeServiceImpl knowledgeApprovalNodeService;

    @Override
    public BaseService getBaseService() {
        return knowledgeApprovalNodeService;
    }

    @Override
    public String getComment() {
        return "业务--知识审批节点";
    }

    @ApiOperation("查询审核流程节点列表")
    @GetMapping("queryApprovalNodes")
    public JsonResult queryApprovalNodes() {
        JsonResult jsonResult = JsonResult.getSuccessResult("获取数据成功");
        List<KnowledgeApprovalVo> approvalNodes = knowledgeApprovalNodeService.queryApprovalNodes();
        jsonResult.setData(approvalNodes);
        return jsonResult;
    }

    @ApiOperation("批量添加审核流程")
    @PostMapping("batchInsertApprovalNodes")
    public JsonResult batchInsertApprovalNodes(@RequestBody List<KnowledgeApprovalNodeDto> knowledgeApprovalNodeDto) {
        JsonResult jsonResult = JsonResult.getSuccessResult("添加数据成功");
        // 清除原有数据
        knowledgeApprovalNodeService.deleteKnowledgeApprovalNode();
        // 批量添加数据
        List<KnowledgeApprovalNode> knowledgeApprovalNodeList = knowledgeApprovalNodeDto
                .parallelStream()
                .map(e -> {
            KnowledgeApprovalNode knowledgeApprovalNode = new KnowledgeApprovalNode();
                    BeanUtils.copyProperties(e, knowledgeApprovalNode);
                    knowledgeApprovalNode.setIsFinal(Integer.parseInt(e.getIsFinal()));
            return knowledgeApprovalNode;
        }).collect(Collectors.toList());
        knowledgeApprovalNodeService.saveBatch(knowledgeApprovalNodeList);
        jsonResult.setData(knowledgeApprovalNodeList);
        return jsonResult;
    }

    @ApiOperation(value = "分页查询审核流程节点")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllApprovalNodes")
    public JsonPageResult findAllApprovalNodes(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = knowledgeApprovalNodeService.findAllApprovalNodes(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }
}

